/* ============================================================================
 * AMES Wholesale Power Market Test Bed (Java): A Free Open-Source Test-Bed  
 *         for the Agent-based Modeling of Electricity Systems 
 * ============================================================================
 *
 * (C) Copyright 2013, by Hongyan Li, Junjie Sun, Sean L. Mooney and Leigh Tesfatsion
 *
 *    Homepage: http://www.econ.iastate.edu/tesfatsi/testMarketHome.htm
 *
 * LICENSING TERMS
 * The AMES Market Package is licensed by the copyright holders (Junjie Sun, 
 * Hongyan Li, Sean L. Mooney, and Leigh Tesfatsion) as free open-source software under the
 * terms of the GNU General Public License (GPL). Anyone who is interested is 
 * allowed to view, modify, and/or improve upon the code used to produce this 
 * package, but any software generated using all or part of this code must be 
 * released as free open-source software in turn. The GNU GPL can be viewed in 
 * its entirety as in the following site: http://www.gnu.org/licenses/gpl.html
 */
package amestest.tests;

// AMES
import amesmarket.*;
import amesmarket.CaseFileReader.BadDataFileFormatException;

import java.io.BufferedReader;

// JAVA
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.CyclicBarrier;
import java.util.logging.Level;
import java.util.logging.Logger;

// JUNIT
import org.junit.*;
import static org.junit.Assert.*;
import uchicago.src.sim.engine.Schedule;

/**
 *
 * @author Sean L. Mooney
 */
public class AMESTest {
    
    protected AMESMarket testMarket;
    protected DefaultSimulationParameters defSimParams;

    @Before
    public void setUp() {
        testMarket = new AMESMarket();
        defSimParams = new DefaultSimulationParameters();
    }
    
    @After
    public void tearDown() {
        testMarket = new AMESMarket();
        defSimParams = new DefaultSimulationParameters();
    }

    @Test
    /**
     * Just test loading the 5 bus example.
     */
    public void testLoad5BusConfig(){
        CaseFileData testConf = load5BusTestCase();

        assertNotNull("Test configuration is null", testConf);
    }

    /**
     * Simple test that the market runs and we get solutions for each day.
     */
    @Test
    public void test5BusExample(){
        CaseFileData testConf = load5BusTestCase();
        assertNotNull("Test configuration is null", testConf);
        InitializeAMESMarket(testConf);
        runTestMarket();

        ArrayList<int[]> solsByDay = testMarket.getHasSolutionByDay();
        assertNotNull("No solution by day found", solsByDay);
        int numSols = solsByDay.size();
        assertEquals("Different number of solutions than max day", testMarket.DAY_MAX, numSols);

        for(int d = 0; d<numSols; d++){
            int[] daySols = solsByDay.get(d);
            for(int h = 0; h < daySols.length; h++) {
                assertEquals("Day " + d + " Hour " + h + " has no solution.",
                        1, daySols[h]);
            }
        }
    }

    /**
     * Run the initialized test market until it finishes.
     * @param testConf
     */
    protected void runTestMarket(){
        //Keep the method from returning until after the market
        //has finished execution.
        final CountDownLatch barrier = new CountDownLatch(1);
        testMarket.addMarketListener(new AMESMarketListener(){
            @Override
            public void marketEvent(MarketEvent e) {
                switch(e) {
                case STOP:
                    barrier.countDown();
                    break;
                }

            }

        });

        testMarket.Start();

        try {
            barrier.await();
        } catch (InterruptedException e1) {
            fail("Interrupted waiting for the market to end!");
        }

    }

    /**
     * Load the 5 bus test case as a test file.
     *
     * May fail if there is an IOExcption thrown when during the loading process.
     */
    public CaseFileData load5BusTestCase(){
        CaseFileData testCase = null;

        try {
            CaseFileReader reader = new CaseFileReader();
            testCase = reader.loadCaseFileData(new File(new File("DATA"), "5BusTestCase.dat"));
        } catch (IOException ex) {
            Logger.getLogger(AMESTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (BadDataFileFormatException ex) {
            Logger.getLogger(AMESTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        }

        return testCase;
    }



    public void InitializeAMESMarket(CaseFileData testConfig) {
        testMarket = new AMESMarket();
        double[][] bus = new double[1][2];
        bus[0][0] = Double.parseDouble(testConfig.nodeData[0][0].toString());
        bus[0][1] = Double.parseDouble(testConfig.nodeData[0][1].toString());

        int iBranchRow = testConfig.branchData.length;
        int iBranchCol = testConfig.branchData[0].length - 1;
        double[][] branch = new double[iBranchRow][iBranchCol];
        for (int i = 0; i < iBranchRow; i++) {
            for (int j = 0; j < iBranchCol; j++) {
                branch[i][j] = Double.parseDouble(testConfig.branchData[i][j + 1].toString());
            }
        }

        int iGenRow = testConfig.genData.length;

        double[][] gen = new double[iGenRow][];
        for (int i = 0; i < iGenRow; i++) {
            Object[] genDataA = testConfig.genData[i].asArray();
            int iGenCol = genDataA.length - 1;
            gen[i] = new double[iGenCol];
            for (int j = 0; j < iGenCol; j++) {
                gen[i][j] = Double.parseDouble(genDataA[j + 1].toString());
            }
        }

        {//Sanity Check. Assert all the rows in gendata have the same length.
            //Every row should have the same number of cols as the first row.
            final int exGenCol = gen[0].length;
            for(int g = 0; g < gen.length; g++) {
                assertEquals(exGenCol, gen[g].length);
            }
        }


        int iLseRow = testConfig.lseData.length;
        int iLseCol = testConfig.lseData[0].length - 1;
        double[][] lse = new double[iLseRow][iLseCol];
        for (int i = 0; i < iLseRow; i++) {
            for (int j = 0; j < iLseCol; j++) {
                lse[i][j] = Double.parseDouble(testConfig.lseData[i][j + 1].toString());
            }
        }

        int iLsePriceRow = testConfig.lsePriceSensitiveDemand.length;
        int iLsePriceCol = testConfig.lsePriceSensitiveDemand[0][0].length - 1;
        double[][][] lsePrice = new double[iLsePriceRow][24][iLsePriceCol];
        for (int i = 0; i < iLsePriceRow; i++) {
            for (int h = 0; h < 24; h++) {
                for (int j = 0; j < iLsePriceCol; j++) {
                    lsePrice[i][h][j] = Double.parseDouble(testConfig.lsePriceSensitiveDemand[i][h][j + 1].toString());
                }
            }
        }

        int iLseHybridRow = testConfig.lseHybridDemand.length;
        int iLseHybridCol = testConfig.lseHybridDemand[0].length - 1;
        int[][] lseHybrid = new int[iLseHybridRow][iLseHybridCol];
        for (int i = 0; i < iLseHybridRow; i++) {
            for (int j = 0; j < iLseHybridCol; j++) {
                lseHybrid[i][j] = Integer.parseInt(testConfig.lseHybridDemand[i][j + 1].toString());
            }
        }

        testMarket.InitLearningParameters(testConfig.genLearningData);

        testMarket.InitSimulationParameters(
                testConfig.iMaxDay,
                testConfig.bMaximumDay,
                testConfig.dThresholdProbability,
                testConfig.bThreshold,
                testConfig.dDailyNetEarningThreshold,
                testConfig.bDailyNetEarningThreshold,
                testConfig.iDailyNetEarningStartDay,
                testConfig.iDailyNetEarningDayLength,
                testConfig.iStartDay,
                testConfig.iCheckDayLength,
                testConfig.dActionProbability,
                testConfig.bActionProbabilityCheck,
                testConfig.iLearningCheckStartDay,
                testConfig.iLearningCheckDayLength,
                testConfig.dLearningCheckDifference,
                testConfig.bLearningCheck,
                testConfig.dGenPriceCap,
                testConfig.dLSEPriceCap,
                testConfig.RandomSeed);

        testMarket.AMESMarketSetupFromGUI(testConfig.baseS, testConfig.baseV, bus, branch, gen, lse, lsePrice, lseHybrid);
    }
}
