/*
 * Created on Aug 20, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package edu.iastate.jrelm.rl;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.core.ActionDomain;

/**
 * For classes that implement reinforcement learning algorithms. 
 * Classes implementing this interface are responsible for 
 * driving the learning process of specific algorithms.
 * 
 * Reinforcement learning algorithms make use of a policy to represent 
 * learned knowledge. Policies themselves require access to the space of possible
 * actions, represented by ActionDomains. As such an ReinforcementLearner will 
 * make use of with a StatelessPolicy and an ActionDomain. 
 * 
 * The types of RLPolicies, ActionDomains, Actions, and ActionIDs can be 
 * parameterized. 
 * 
 * @author Charles Giseler
 *
 */

public interface ReinforcementLearner
		<I, A extends Action, R, P extends Policy> {

	/**
	 * Initiate the learning process using given feedback. Feedback is associated 
	 * with a the last Action chosen by this engine and is interpreted as a reward
	 * for that Action. It is used to update the probability for choosing the Action 
	 * according to the specific learning algorithm.
	 * <P>
	 * Feedback is parameterized since required input will vary depending on the 
	 * specific reinforcement learning algorithm and the particular simulation 
	 * environment. 
	 * <P>
	 * Note: Most often feedback is for the last Action chosen, so given ActionID 
	 * will usually point to this Action. As such, many RLEnigine implementations
	 * may also provide update() methods that simply accept feedback and associate 
	 * it with the last Action chosen.
	 * 
	 * @param reward - feedback for the specified action
	 */
	public void update(R reward);
	
	/**
	 * Elicits a new choice of action. The action will be chosen according to 
	 * selection rule of the SimpleStatelessPolicy. Actions are chosen from a 
	 * DiscreteFiniteDomain.
	 * 
	 * @return the next action chosen. Selected actions can be any object 
	 * 		   implementing the Action interface. 
	 * @see edu.iastate.jrelm.core.Action
	 */
	public A chooseAction();
	
	/**
	 * Retrieve the RLParameters that contain settings for this learning 
	 * algorithm.
	 * 
	 * @return learning algorithm settings as RLParameters
	 */
	public RLParameters getParameters();
	
	/**
	 * Create a default set of parameters that can be used with this learner.
	 * 
	 * @return learning parameters compatible with this learner, initialized 
	 * to default settings. 
	 */
	public RLParameters makeParameters();
	
	/**
	 * Sets the current settings for this learning algorithm.
	 *
	 * @param parameters - new settings for this algorithm as RLParameters
	 */
	public void setParameters(RLParameters params);
	
	/**
	 * Retrieve the StatelessPolicy being used to represent learned knowledge.
	 * 
	 * @return the Policy being used by this ReinforcementLearner. The policy can be any object
	 * 		   implementing the StatelessPolicy interface.
	 * @see edu.iastate.jrelm.rl.StatelessPolicy
	 */
	public P getPolicy();
	
	/**
	 * Set the StatelessPolicy to be used to represent learned knowledge.
	 * 
	 * @param p - The policy can be any object implementing the StatelessPolicy interface.
	 * @see edu.iastate.jrelm.rl.StatelessPolicy
	 */
	public void setPolicy(P policy);
	
	/**
	 * Retrieves the name of the learning algorithm this learner implements.
	 * @return - the algorithm name
	 */
	public String getName();
	
}
