package edu.iastate.jrelm.rl.rotherev.advanced;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.similarity.DissimilarityMeasure;
import edu.iastate.jrelm.spillover.SpilloverByDissimilarity;
import edu.iastate.jrelm.spillover.SpilloverWeightingException;

/**
 * Implements a simple linear weight function of the form
 * 
 * 			a * x + 1
 * 
 * where x is a value indicating the dissimilarity of two Actions as determined 
 * by the given DissimilarityMeasure
 * <p>
 * Note: The VRELearner uses weight values between 0 and 1. As such, 'a' 
 * must be set to a value <= 0 and the y intercept is fixed at 1. 
 * 
 *  
 * @author Charles Gieseler
 *
 * @param <A> - The type of Action 
 * @param <D> - The type of DissimilarityMeasure to use.
 * <p>
 * Note: D is restricted to DissimilarityMeasures 
 * that return Doubles as indicators.
 */
public class LinearSpillover
			<A extends Action, 
			 D extends DissimilarityMeasure<A, Double>> 
			extends SpilloverByDissimilarity<A, D> {

	private double paramA = -1;
	
	/**
	 * Construct a linear weighing method with the given similarity 
	 * measure and parameter value.
	 * <P>
	 * Note: the method of comparison will automatically be set to use dissimilarity 
	 * rather than similarity.
	 * 
	 * @param measure - the DissimilarityMeasure to use for comparison of Actions
	 * @param a - parameter for adjusting the slope
	 * 
	 */
	public LinearSpillover(D measure, double a) {
		super(measure);
		
		try{
			this.setParamA(a);
		}catch(SpilloverWeightingException swe){
			System.err.println("Received an illegal value for parameter 'a' " +
					"in LinearSpillover. Using default value of -1.");
			System.err.println(swe.getMessage());
			
		}
	}
	
	/**
	 * Generates a reward spillover weight using an exponential function
	 * 
	 * 			a * x + b
	 * 
	 * where x is a numerical (Double) indicator of how dissimilar the given Action
	 * is to the reference Action. 
	 */
	
	public double generateWeight(A action) {
		double weight = 0;
		Double dissimilarityValue;
		double x;

		//Compare the given Action with the reference Action to
		//determine how similar they are
		A refAction = getReferenceAction();
		dissimilarityValue = getDissimilarityMeasure().dissimilarity(refAction, action);
		x = dissimilarityValue.doubleValue();
		
		//Calculate the weight value
		weight = paramA * x + 1;
		
		return weight;
	}

	public String getName(){return "Linear Weighting";}
	
	public double getParamA() {
		return paramA;
	}
	
	/**
	 * Set the value of parameter 'a'.
	 * 
	 * Note: The VRELearner uses weight values between 0 and 1. As such, 'a' 
	 * must be set to a value <= 0.
	 * 
	 * @param aValue - desired value for parameter 'b'
	 */
	public void setParamA(double aValue) throws SpilloverWeightingException {
		if(aValue <= 0)
			this.paramA = aValue;
		else
			throw new SpilloverWeightingException("Illegal value for parameter 'a' in " +
					"LinearSpillover: " + aValue +". Must set 'a' to a value <= 0.");
	}

}
	

