/*
/* Created on Jun 17, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

package edu.iastate.jrelm.rl.rotherev.variant;

import java.util.ArrayList;

import edu.iastate.jrelm.core.Action;
import edu.iastate.jrelm.core.ActionDomain;
import edu.iastate.jrelm.rl.RLParameters;
import edu.iastate.jrelm.rl.ReinforcementLearner;
import edu.iastate.jrelm.rl.rotherev.RELearner;
import edu.iastate.jrelm.rl.rotherev.advanced.AREParameters;
import edu.iastate.jrelm.rl.rotherev.variant.VREParameters;
import edu.iastate.jrelm.rl.rotherev.REPolicy;

import static java.lang.System.out;
/**
 * Variant Roth-Erev Learner
 * 
 * This ReinforcementLearner implements a variation of the Roth-Ere'ev algorithm 
 * as presented in <br>
 *    James Nicolaisen, Valentin Petrov, and Leigh Tesfatsian, "Market Power and <br> 
 *    Efficiency in a Computational Electricity Market with Discriminatory <br>
 *    Double-Auction Pricing," IEEE Transactions on Evolutionary Computation, <br>
 *    Volume 5, Number 5, 2001, 504-523. 
 *
 * See RELearner for details on the original Roth-Erev algorithm
 * 
 * @author Charles Gieseler
 *
 */
public class VRELearner<I, A extends Action> 
		 extends RELearner<I, A> {
	
	private VREParameters parameters;
	
	/* ************************************************************************************
	 * 						 			CONSTRUCTORS
	 * ************************************************************************************/
	/**
	 * Construct a learner using Variant Roth-Erev with parameters specified in a
	 * VREParameters object. 
	 * 
	 * @param learningParams - the collection of parameter settings for the 
	 * 						VRELearner
	 * @param aDomain - the ActionDomain to learn over
	 */
	public VRELearner(VREParameters learningParams, ActionDomain<I,A> aDomain){
		super(learningParams, aDomain);
		parameters = learningParams;
	}
	
	/**
	 * Construct a RothErev learning component with parameters specified in a
	 * AMREParameters object and the given policy. 
	 * 
	 * Note: any random seed that is already set in the given policy will be 
	 * overwritten with the seed in the given parmaters.  
	 * 
	 * @param learningParams - the collection of parameter settings for the 
	 * 						VRELearner
	 * @param aPolicy - a SimpleStatelessPolicy
	 */
	public <P extends REPolicy<I,A>> VRELearner(VREParameters learningParams, 
			P aPolicy){
		super(learningParams, aPolicy);	
		parameters = learningParams;

	}

	
	/*/***********************************************************************************
	 * 							ACTION PROBABILITY UPDATING
	 ************************************************************************************/	
	/*
	 * Updates the probability for each action to be chosen in the policy.  
	 * 
	 */
	protected void updateProbabilities(){
		generateBoltzmanProbs();
	}
	
	/*
	 * Generate action probabilities using a Boltzmann distribution with a constant
	 * temperature
	 * 
	 */
	protected void generateBoltzmanProbs(){

		//Do this because the super class will try to generate probs before 
		//the new parameters are in place. Check for policy just in case 
		if(parameters == null || policy == null)
			return;

		double[] propensities = policy.getPropensities();
		double coolingParam = parameters.getBoltzmannTemp();
		double summedExps = 0;
		double newProb;
		
		
		//Sum all of the exponentials of the the propensities 
		for(double prop: propensities)
			summedExps = summedExps + Math.exp((prop/coolingParam));
		
		//For each action calculate the associated choice probability 
		//	p(i) = [ e ^(q(i)/T) ] / [ Sum_over_all_j(e ^ (q(j)/T)) ]
		int index;
		for(I actID: actionIDList){
			index = actionIDList.indexOf(actID);
			newProb = Math.exp((propensities[index]/coolingParam)) / summedExps;
			policy.setProbability(actID, newProb);
		}
	}
	
	/*/************************************************************************************
	 * 						 		RESPONSE GENERATION
	 **************************************************************************************/

	/*
	 * This is an altered version of the experience function for used in the standard
	 * Roth-Erev algorithm.  Like in RELearner, propensities for all actions are updated and similarity does not 
	 * come into play. If the actionIndex points to the action the reward is associated with 
	 * (usually the last action taken) then simply adjust the weight by the experimentation. 
	 * Otherwise increase the weight of the action by a small portion of its current propensity.
	 * This puts the "Modified" in Modified Roth-Erev algorithm. 
	 *  
	 *  If j is the index of the last action chosen, r_j is the reward received for performing j, 
	 *  i is the current action being updated, q_i is the propensity for i, n is the size 
	 *  of the action domain and e is the experimentation parameter, then this experience function 
	 *  can be expressed as
	 *   			 _
	 *  			|  r_j * (1-e) 		if i = j
	 *  E(i, r_j) = | 
	 *				|_ q_i * (e /(n-1))	if i != j
	 *
	 *
	 */				
	 				
	protected double experience(int actionIndex, double reward){
//System.out.println("VRELearner.exeperience(): actionIndex: "+actionIndex+" reward: "+reward);		
		double experience  = 0;
		double e = parameters.getExperimentation();
//out.println("	lastSelectedAction: "+actionIndex;
//out.println("	actionIDList contains lastAction: "+ actionIDList.contains(actionIndex));
		int rewardedIndex = actionIDList.indexOf(lastSelectedAction.getID());
//out.println("	rewardedIndex: "+ rewardedIndex);
		//if this action is the last a
		if(actionIndex == rewardedIndex){
			experience = reward * (1-e);
		}else{
//out.println("	prop: "+policy.getPropensity(actionIndex)+ " experiment update: " + (e /(domainSize - 1)));			
			experience = policy.getPropensity(actionIndex) * (e /(domainSize - 1));
		}
//out.println("	experience: "+ experience);		
		return experience;
	}
	
	
	public String getName() {
		return "Modified Roth-Erev";
	}
}
