package edu.iastate.jrelm.similarity;

import java.util.Hashtable;
import java.util.Map;

import edu.iastate.jrelm.core.Action;

/**
 * This class is for building a mapping of Action pairs to similarity indicators. This is useful 
 * for situations where implementing functions to calculate similarity is difficult or impossible.
 * Similarity map can be built incrementally by adding inividual mappings or can be initialized
 * off of a given mapping in the form of a Hashtable. 
 * <p>
 * One advantage of using a SimilarityMap is that it can easily be expanded or redefined by adding,
 * removing and modifying entries. However, one must be careful to keep it in sync with the 
 * ActionDomain being used. Attempts to compare Actions that are not in the map will fail.
 * 
 * @author Charles Gieseler
 *
 * @param <A> - The type of Action this measure can compare.
 * @param <O> - The type that is returned to indicate similarity
 */
public class SimilarityMap<A extends Action, O> 
		implements SimilarityMeasure<A, O>{

	Hashtable<Action[], O> map;
	
	public SimilarityMap(){
		map = new Hashtable<Action[], O>();
	}

	public SimilarityMap(Hashtable<Action[], O> similarityTable){
		map = similarityTable;
	}
	
	
	public void setMapping(A action1, A action2, O similarityIndicator){
		Action[] actionPairKey = {action1, action2};
		map.put(actionPairKey, similarityIndicator);
	}
	
	public void setMapping(Map<Action[], O> newMapping){
		map.putAll(newMapping);
	}
	
	/**
	 * Access the stored similarity indicator for these
	 */
	public O similarity(A action1, A action2) {
		Action[] actionPairKey = {action1, action2};	
		return map.get(actionPairKey);
	}
	
	public void removeMapping(A action1, A action2){
		Action[] actionPairKey = {action1, action2};
		map.remove(actionPairKey);
	}
	
	public int size(){return map.size();}
}
