package edu.iastate.jrelm.similarity;

import edu.iastate.jrelm.core.Action;

/**
 * This interface is for constructing classes that can compare two Actions and determine
 * how similar they are. Such similarity measures are used by learning algorithms that 
 * need to determine how actions relate to one another. An example of this is the spill 
 * over reward value among Actions similar to the last Action chosen.
 * <P>
 * Which Action properties are compared and how similarity is defined can be customized 
 * for particular simulation contexts. In addition, measures can be built to return a 
 * varity of different similarity indicators. For example, a measure could return a 
 * simple Boolean to indicate the two actions meet the similarity requirements. Another
 * could return a range of Doubles indicating degrees of similarity.
 * 
 * @author Charles Gieseler
 *
 * @param <A> - The type of Action this measure can compare.
 * @param <O> - The type that is returned to indicate similarity
 */
public interface SimilarityMeasure<A extends Action, O> {

	/**
	 * Performs a comparison of two Action objects and determines how similar the two are.
	 * 
	 * @param action1 - First Action.
	 * @param action2 - Second Action.
	 * @return an object representing the measure of similarity between action 1 and action2.
	 */
	public O similarity(A action1, A action2);
	
}
