package edu.iastate.jrelm.similarity;

import edu.iastate.jrelm.core.Action;

/**
 * This is an abstract class for building similarity measures that return double values between
 * 0 and 1. These bounds may be inclusive of 0 or 1 or both, as specified by particular similarity
 * measures extending this class. 
 * <P> 
 * Normally classes implementing the SimilarityMeasure interface check for similarity by 
 * implementing the similarity() method. However, this class requires that comparison of 
 * similarity be done by implementing the abstract method assessSimilarity(). The similarity()
 * method calls assessSimilarity() and checks to make sure the value it returns is between 0 and
 * 1.
 * 
 * @author Charles Gieseler
 *
 * @param <A> - the type of Action that this measure compares.
 */
public abstract class ZeroOneBoundedSimilarity<A extends Action> 
		implements SimilarityMeasure<A, Double> {

	/*
	 * Checks to make sure a given similarity value is between 0 and 1 inclusive
	 */
	private boolean valueInBounds(double simValue){
		if((simValue >= 0) && (simValue <= 1))
			return true;
		else
			return false;
	}
	
	/**
	 * Similarity measures extending ZeroOneBoundedSimilarity must implement this method 
	 * to perform the actual comparison of two given Actions. The degree of similarity should
	 * be indicated by a double value between 0 and 1 inclusive. These bounds restrictions 
	 * are checked by the similarity() method, which calls on assessSimilarity() to make the 
	 * comparison. Further restrictions on the bounds may be imposed by specific measures.
	 * <p>
	 * In actual use, the similarity() method is called to intiate the comparison process. 
	 * 
	 * @param action1 - First Action object
	 * @param action2 - Second Action object
	 * @return value between 0 and 1 indicating the degree of similarity between action1 
	 *			and action2 
	 */
	protected abstract double assesSimilarity(A action1, A action2);
	
	/**
	 * Performs a comparison of two Action objects and determines how similar the two are. 
	 * 
	 */
	public Double getMeasure(A action1, A action2){
		
		double simValue = assesSimilarity(action1, action2);
		
		//Check to make sure the similarity value is within bounds
		if(!valueInBounds(simValue)){
			String message = "Attempted to set a similarity value out of bounds. \n";
			if(simValue < 0){
				message = message + simValue + " is less than 0. ";
			}else if(simValue > 1){
				message = message + simValue + " is greater than 1.";
			}
			System.err.println(message);
			
			//Return an invalid similarity measure.
			return Double.NaN;
			
		}else{
			//Value has passed the bounds check, so return it as a valid measure
			return simValue;
		}
	}
}
